/*****************************************************************************
*           Change Log
*  Date     | Change
*-----------+-----------------------------------------------------------------
* 30-Aug-99 | [1.384] Created change log
* 30-Aug-99 | [1.384] Added RegistryGUID
* 26-Dec-99 | [1.387] REQ #134: Minor changes to handle Lint warnings
*  5-May-04 | Rework. Values are BOOL. Added types for DWORD arrays, fonts, etc.
*****************************************************************************/

class RegistryVar {
    public:
        static BOOL setInt(UINT id, DWORD value, HKEY root = HKEY_CURRENT_USER);
        static BOOL getInt(UINT id, DWORD & value, DWORD def = 0, HKEY root = HKEY_CURRENT_USER);

        static BOOL setString(UINT id, LPCTSTR s, HKEY root = HKEY_CURRENT_USER);
        static BOOL getString(UINT id, CString & s, LPCTSTR def = NULL, HKEY root = HKEY_CURRENT_USER);

        static BOOL setGUID(UINT id, const GUID & value, HKEY root = HKEY_CURRENT_USER);
        static BOOL getGUID(UINT id, GUID & result, const GUID * def, HKEY root = HKEY_CURRENT_USER);

        static BOOL setInt64(UINT id, ULONGLONG value, HKEY root = HKEY_CURRENT_USER);
        static BOOL getInt64(UINT id, ULONGLONG & value, ULONGLONG def, HKEY root = HKEY_CURRENT_USER);

        static BOOL setFont(UINT id, LOGFONT * value, HKEY root = HKEY_CURRENT_USER);
        static BOOL getFont(UINT id, LOGFONT & result, LOGFONT * def = NULL, HKEY root = HKEY_CURRENT_USER);

        static BOOL setWindowPlacement(UINT id, WINDOWPLACEMENT * value, HKEY root = HKEY_CURRENT_USER);
        static BOOL getWindowPlacement(UINT id, WINDOWPLACEMENT & result, WINDOWPLACEMENT * def = NULL, HKEY root = HKEY_CURRENT_USER);
    protected:
        RegistryVar(UINT n, HKEY r) { id = n; root = r; }
        UINT id;
        HKEY root;
               };

/****************************************************************************
*                                 RegistryInt
****************************************************************************/

/*lint -e1735 */
class RegistryInt : public RegistryVar {
     public:
        RegistryInt(UINT n, HKEY r = HKEY_CURRENT_USER ) : RegistryVar(n, r) { value = 0; } // REQ #134
        virtual BOOL load(DWORD def = 0);
        virtual BOOL store();

        DWORD value; // the value
                                       };
            

/****************************************************************************
*                                RegistryString
****************************************************************************/

class RegistryString : public RegistryVar {
     public:
        RegistryString(UINT n, HKEY r = HKEY_CURRENT_USER) : RegistryVar(n, r) { }
        virtual BOOL load(LPCTSTR def = NULL);
        virtual BOOL store();

        CString value; // the value
};

/****************************************************************************
*                                 RegistryGUID
****************************************************************************/
                                       
#ifdef GUID_DEFINED
class RegistryGUID : public RegistryVar {
    public:
       RegistryGUID(UINT n, HKEY r = HKEY_CURRENT_USER) : RegistryVar(n, r) { ::ZeroMemory(&value, sizeof(value)); }
       virtual BOOL load(const GUID * def = NULL);
       virtual BOOL store();

       GUID value; // the value
};
#endif

/****************************************************************************
*                                 RegistryFont
****************************************************************************/
class RegistryFont : public RegistryVar {
    public:
       RegistryFont(UINT n, HKEY r = HKEY_CURRENT_USER) : RegistryVar(n, r) { ::ZeroMemory(&value, sizeof(value)); }
       virtual BOOL load(const LOGFONT * def = NULL);
       virtual BOOL store();

       LOGFONT value;
};

/****************************************************************************
*                           RegistryWindowPlacement
****************************************************************************/
class RegistryWindowPlacement : public RegistryVar {
    public:
       RegistryWindowPlacement(UINT n, HKEY r = HKEY_CURRENT_USER) : RegistryVar(n, r) { ::ZeroMemory(&value, sizeof(value)); }
       virtual BOOL load(const WINDOWPLACEMENT * def = NULL);
       virtual BOOL store();

       WINDOWPLACEMENT value;
};
